package java.io;

import sun.nio.ch.FileChannelImpl;
import java.nio.channels.FileChannel;

public class FileOutputStream extends OutputStream
{
    private final FileDescriptor fd;
    private final boolean append;
    private FileChannel channel;
    private final String path;
    private final Object closeLock;
    private volatile boolean closed;
    
    public FileOutputStream(String s) throws FileNotFoundException {
        this((s != null) ? new File(s) : null, false);
    }
    
    public FileOutputStream(String s, boolean b) throws FileNotFoundException {
        this((s != null) ? new File(s) : null, b);
    }
    
    public FileOutputStream(File file) throws FileNotFoundException {
        this(file, false);
    }
    
    public FileOutputStream(File file, boolean append) throws FileNotFoundException {
        this.closeLock = new Object();
        this.closed = false;
        String s = (file != null) ? file.getPath() : null;
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkWrite(s);
        }
        if (s == null) {
            throw new NullPointerException();
        }
        if (file.isInvalid()) {
            throw new FileNotFoundException("Invalid file path");
        }
        (this.fd = new FileDescriptor()).attach(this);
        this.append = append;
        this.open(this.path = s, append);
        return;
    }
    
    public FileOutputStream(FileDescriptor fileDescriptor) {
        this.closeLock = new Object();
        this.closed = false;
        SecurityManager securityManager = System.getSecurityManager();
        if (fileDescriptor == null) {
            throw new NullPointerException();
        }
        if (securityManager != null) {
            securityManager.checkWrite(fileDescriptor);
        }
        this.fd = fileDescriptor;
        this.append = false;
        this.path = null;
        this.fd.attach(this);
        return;
    }
    
    private native void open0(String p0, boolean p1) throws FileNotFoundException;
    
    private void open(String s, boolean b) throws FileNotFoundException {
        this.open0(s, b);
    }
    
    private native void write(int p0, boolean p1) throws IOException;
    
    @Override
    public void write(int n) throws IOException {
        this.write(n, this.append);
    }
    
    private native void writeBytes(byte[] p0, int p1, int p2, boolean p3) throws IOException;
    
    @Override
    public void write(byte[] array) throws IOException {
        this.writeBytes(array, 0, array.length, this.append);
    }
    
    @Override
    public void write(byte[] array, int n, int n2) throws IOException {
        this.writeBytes(array, n, n2, this.append);
    }
    
    @Override
    public void close() throws IOException {
        synchronized (this.closeLock) {
            if (this.closed) {
                return;
            }
            this.closed = true;
        }
        if (this.channel != null) {
            this.channel.close();
        }
        this.fd.closeAll(new Closeable() {
            @Override
            public void close() throws IOException {
                FileOutputStream.access$000(FileOutputStream.this);
            }
        });
    }
    
    public final FileDescriptor getFD() throws IOException {
        if (this.fd != null) {
            return this.fd;
        }
        throw new IOException();
    }
    
    public FileChannel getChannel() {
        synchronized (this) {
            if (this.channel == null) {
                this.channel = FileChannelImpl.open(this.fd, this.path, false, true, this.append, this);
            }
            return this.channel;
        }
    }
    
    @Override
    protected void finalize() throws IOException {
        if (this.fd != null) {
            if (this.fd == FileDescriptor.out || this.fd == FileDescriptor.err) {
                this.flush();
            }
            else {
                this.close();
            }
        }
    }
    
    private native void close0() throws IOException;
    
    private static native void initIDs();
    
    static void access$000(FileOutputStream fileOutputStream) throws IOException {
        fileOutputStream.close0();
    }
    
    static {
        initIDs();
    }
}
